<?php

/**
 * @file
 * Apache Solr Search Integration hook implementations and helper functions.
 */

/**
 * Converts the Solr field containing the image to the info field name.
 *
 * @param string $field_name
 *   The name of the image field as it is stored in Solr.
 *
 * @return string
 *   The name of the image info field as it is stored in Solr.
 */
function rich_snippets_get_apachesolr_image_info_field($field_name) {
  $base_name = substr($field_name, 3);
  return 'zm_' . $base_name . '_info';
}

/**
 * Implements hook_apachesolr_query_alter().
 *
 * Include all fields that have associated schema.
 */
function rich_snippets_apachesolr_query_alter($query) {
  $solr_fields = array();

  // Add highlighting to the title.
  $query->addParam('hl.fl', 'label');

  // Load the Apache Solr Search Integration API functions if they don't exist.
  if (!function_exists('apachesolr_get_field_mappings')) {
    module_load_include('inc', 'rich_snippets', 'rich_snippets.apachesolr_legacy');
  }

  // Load the environment associated with this query.
  $env_id = $query->solr('getId');
  $enviromnent = apachesolr_environment_load($env_id);

  // For all bundles indexed by this environment, gather a list of Solr fields
  // that will need to be returned by the Solr server so we can append them to
  // the rich snippets.
  foreach ($enviromnent['index_bundles'] as $entity_type => $bundles) {
    $key_mappings = apachesolr_get_index_key_map($entity_type);
    foreach ($bundles as $bundle) {
      $rdf_mappings = rich_snippets_get_rdf_schema_mappings($entity_type, $bundle);
      foreach ($rdf_mappings as $schema => $fields) {
        foreach ($fields as $field) {
          if (0 === strpos($field, 'field_') && isset($key_mappings[$field])) {

            // Map the fiel to the corresponsding field in Solr.
            $solr_fields[$field] = array($key_mappings[$field]);

            // Assume that images map to an image field, so add the info field.
            if ('image' == $schema) {
              $solr_fields[$field][] = rich_snippets_get_apachesolr_image_info_field($key_mappings[$field]);
            }
          }
        }
      }
    }
  }

  // Add each field to the list of returned fields.
  foreach ($solr_fields as $field_names) {
    foreach ($field_names as $field_name) {
      $query->addParam('fl', $field_name);
    }
  }

  // Add the field storing the schema mappings.
  $query->addParam('fl', 'zm_rdf_schema_mappings');
}

/**
 * Implements hook_apachesolr_search_result_alter().
 */
function rich_snippets_apachesolr_search_result_alter($doc, $extra, $query) {
  $response = apachesolr_static_response_cache($query->getSearcher());
  if (!empty($response->highlighting->{$doc->id}->label)) {
    $doc->label = '';
    $highlighted = $response->highlighting->{$doc->id}->label;
    foreach ($highlighted as $text) {
      $doc->label .= $text;
    }
  }
}

/**
 * Implements hook_apachesolr_field_mappings().
 */
function rich_snippets_apachesolr_field_mappings() {
  $mappings = array();

  // @todo Store in binary instead?
  $mappings['image'] = array(
    'indexing_callback' => 'rich_snippets_apachesolr_image_indexing_callback',
    'index_type' => 'binary',
    'facets' => FALSE,
  );

  $mappings['text'] = array(
    'indexing_callback' => 'apachesolr_fields_default_indexing_callback',
    'index_type' => 'text',
    'facets' => FALSE,
  );

  return $mappings;
}

/**
 * Indexing callback that stores the image uri.
 */
function rich_snippets_apachesolr_image_indexing_callback($entity, $field_name, $index_key, $field_info) {
  $fields = array();
  if (!empty($entity->{$field_name})) {
    $field = $entity->$field_name;
    list($lang, $values) = each($field);
    for ($i = 0; $i < count($values); $i++) {

      $image_uri = image_style_path('rich_snippets_thumbnail', $values[$i]['uri']);
      if ($image_uri && ($file = image_load($image_uri))) {
        if ($data = file_get_contents($image_uri)) {

          // Store the base64 encoded image data.
          $fields[] = array(
            'key' => $index_key,
            'value' => base64_encode($data),
          );

          // Store the image info in an unindexed text field.
          $fields[] = array(
            'key' => rich_snippets_get_apachesolr_image_info_field($index_key),
            'value' => drupal_json_encode($file->info),
          );
        }
      }
    }
  }
  return $fields;
}

/**
 * Implements hook_apachesolr_index_document_build().
 *
 * Adds schema.org mappings to the index.
 */
function rich_snippets_apachesolr_index_document_build(ApacheSolrDocument $document, $entity, $entity_type, $env_id) {
  if ($bundle = rich_snippets_extract_bundle($entity_type, $entity)) {
    $rdf_mappings = rich_snippets_get_rdf_schema_mappings($entity_type, $bundle);
    $document->setField('zm_rdf_schema_mappings', drupal_json_encode($rdf_mappings));
  }
}

/**
 * Helper function to get the entity's bundle.
 *
 * Is there really no API function to get an entity's bundle? I must have
 * missed it.
 *
 * @param string $entity_type
 *   The machine name of the entity.
 * @param stdClass $entity
 *   The entity the bundle is being extracted from.
 *
 * @return string|FALSE
 *   The machine name of the bundle, FALSE if the bundle couldn't be extracted.
 */
function rich_snippets_extract_bundle($entity_type, $entity) {
  $bundle = FALSE;
  if ($entity_info = entity_get_info('node')) {
    // Attempt to extract the bundle from the entity keys. Make sure that the
    // entity we extracted is valid just in case.
    foreach ($entity_info['bundle keys'] as $key) {
      if (isset($entity->$key) && isset($entity_info['bundles'][$entity->$key])) {
        $bundle = $entity->$key;
        break;
      }
    }
  }
  return $bundle;
}
